@@ -28,7 +28,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
-#endif /* HAVE_CONFIG_H */
+#endif /* defined(HAVE_CONFIG_H) */
 
 #include <nghttp3/nghttp3.h>
 
@@ -48,6 +48,14 @@
 /* NGHTTP3_QPACK_MAX_VALUELEN is the maximum (compressed) length of
    header value this library can decode. */
 #define NGHTTP3_QPACK_MAX_VALUELEN 65536
+/* NGHTTP3_QPACK_MAX_ENCODERLEN is the maximum encoder stream length
+   that a decoder accepts without completely processing a single field
+   section. */
+#define NGHTTP3_QPACK_MAX_ENCODERLEN (128 * 1024)
+/* NGHTTP3_QPACK_MAX_DECODERLEN is the maximum decoder stream length
+   that an encoder accepts without completely encoding a single field
+   section. */
+#define NGHTTP3_QPACK_MAX_DECODERLEN (4 * 1024)
 
 /* nghttp3_qpack_indexing_mode is a indexing strategy. */
 typedef enum nghttp3_qpack_indexing_mode {
@@ -250,6 +258,9 @@ struct nghttp3_qpack_encoder {
   /* last_max_dtable_update is the dynamic table size last
      requested. */
   size_t last_max_dtable_update;
+  /* uninterrupted_decoderlen is the number of bytes read from decoder
+     stream without encoding a single field section. */
+  size_t uninterrupted_decoderlen;
   /* flags is bitwise OR of zero or more of
      NGHTTP3_QPACK_ENCODER_FLAG_*. */
   uint8_t flags;
@@ -259,16 +270,10 @@ struct nghttp3_qpack_encoder {
  * nghttp3_qpack_encoder_init initializes |encoder|.
  * |hard_max_dtable_capacity| is the upper bound of the dynamic table
  * capacity.  |mem| is a memory allocator.
- *
- * This function returns 0 if it succeeds, or one of the following
- * negative error codes:
- *
- * NGHTTP3_ERR_NOMEM
- *     Out of memory.
  */
-int nghttp3_qpack_encoder_init(nghttp3_qpack_encoder *encoder,
-                               size_t hard_max_dtable_capacity,
-                               const nghttp3_mem *mem);
+void nghttp3_qpack_encoder_init(nghttp3_qpack_encoder *encoder,
+                                size_t hard_max_dtable_capacity,
+                                const nghttp3_mem *mem);
 
 /*
  * nghttp3_qpack_encoder_free frees memory allocated for |encoder|.
@@ -325,9 +330,9 @@ nghttp3_qpack_lookup_stable(const nghttp3_nv *nv, int32_t token,
  * blocked (or it has been blocked already).
  */
 nghttp3_qpack_lookup_result nghttp3_qpack_encoder_lookup_dtable(
-    nghttp3_qpack_encoder *encoder, const nghttp3_nv *nv, int32_t token,
-    uint32_t hash, nghttp3_qpack_indexing_mode indexing_mode, uint64_t krcnt,
-    int allow_blocking);
+  nghttp3_qpack_encoder *encoder, const nghttp3_nv *nv, int32_t token,
+  uint32_t hash, nghttp3_qpack_indexing_mode indexing_mode, uint64_t krcnt,
+  int allow_blocking);
 
 /*
  * nghttp3_qpack_encoder_write_field_section_prefix writes Encoded
@@ -341,8 +346,8 @@ nghttp3_qpack_lookup_result nghttp3_qpack_encoder_lookup_dtable(
  *     Out of memory.
  */
 int nghttp3_qpack_encoder_write_field_section_prefix(
-    nghttp3_qpack_encoder *encoder, nghttp3_buf *pbuf, uint64_t ricnt,
-    uint64_t base);
+  nghttp3_qpack_encoder *encoder, nghttp3_buf *pbuf, uint64_t ricnt,
+  uint64_t base);
 
 /*
  * nghttp3_qpack_encoder_write_static_indexed writes Indexed Header
@@ -386,8 +391,8 @@ int nghttp3_qpack_encoder_write_dynamic_indexed(nghttp3_qpack_encoder *encoder,
  *     Out of memory.
  */
 int nghttp3_qpack_encoder_write_static_indexed_name(
-    nghttp3_qpack_encoder *encoder, nghttp3_buf *rbuf, uint64_t absidx,
-    const nghttp3_nv *nv);
+  nghttp3_qpack_encoder *encoder, nghttp3_buf *rbuf, uint64_t absidx,
+  const nghttp3_nv *nv);
 
 /*
  * nghttp3_qpack_encoder_write_dynamic_indexed writes Literal Header
@@ -402,8 +407,8 @@ int nghttp3_qpack_encoder_write_static_indexed_name(
  *     Out of memory.
  */
 int nghttp3_qpack_encoder_write_dynamic_indexed_name(
-    nghttp3_qpack_encoder *encoder, nghttp3_buf *rbuf, uint64_t absidx,
-    uint64_t base, const nghttp3_nv *nv);
+  nghttp3_qpack_encoder *encoder, nghttp3_buf *rbuf, uint64_t absidx,
+  uint64_t base, const nghttp3_nv *nv);
 
 /*
  * nghttp3_qpack_encoder_write_literal writes Literal Header Field
@@ -779,6 +784,9 @@ struct nghttp3_qpack_decoder {
      unidirectional streams which potentially receives QPACK encoded
      HEADER frame. */
   size_t max_concurrent_streams;
+  /* uninterrupted_encoderlen is the number of bytes read from encoder
+     stream without completing a single field section. */
+  size_t uninterrupted_encoderlen;
 };
 
 /*
@@ -786,17 +794,11 @@ struct nghttp3_qpack_decoder {
  * |hard_max_dtable_capacity| is the upper bound of the dynamic table
  * capacity.  |max_blocked_streams| is the maximum number of stream
  * which can be blocked.  |mem| is a memory allocator.
- *
- * This function returns 0 if it succeeds, or one of the following
- * negative error codes:
- *
- * NGHTTP3_ERR_NOMEM
- *     Out of memory.
  */
-int nghttp3_qpack_decoder_init(nghttp3_qpack_decoder *decoder,
-                               size_t hard_max_dtable_capacity,
-                               size_t max_blocked_streams,
-                               const nghttp3_mem *mem);
+void nghttp3_qpack_decoder_init(nghttp3_qpack_decoder *decoder,
+                                size_t hard_max_dtable_capacity,
+                                size_t max_blocked_streams,
+                                const nghttp3_mem *mem);
 
 /*
  * nghttp3_qpack_decoder_free frees memory allocated for |decoder|.
@@ -991,6 +993,6 @@ void nghttp3_qpack_decoder_emit_literal(nghttp3_qpack_decoder *decoder,
  *     Decoder stream overflow.
  */
 int nghttp3_qpack_decoder_write_section_ack(
-    nghttp3_qpack_decoder *decoder, const nghttp3_qpack_stream_context *sctx);
+  nghttp3_qpack_decoder *decoder, const nghttp3_qpack_stream_context *sctx);
 
-#endif /* NGHTTP3_QPACK_H */
+#endif /* !defined(NGHTTP3_QPACK_H) */
