@@ -27,7 +27,7 @@
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
-#endif /* HAVE_CONFIG_H */
+#endif /* defined(HAVE_CONFIG_H) */
 
 #include <nghttp3/nghttp3.h>
 
@@ -50,13 +50,13 @@
 #define NGHTTP3_STREAM_MIN_WRITELEN 800
 
 /* nghttp3_stream_type is unidirectional stream type. */
-typedef enum nghttp3_stream_type {
-  NGHTTP3_STREAM_TYPE_CONTROL = 0x00,
-  NGHTTP3_STREAM_TYPE_PUSH = 0x01,
-  NGHTTP3_STREAM_TYPE_QPACK_ENCODER = 0x02,
-  NGHTTP3_STREAM_TYPE_QPACK_DECODER = 0x03,
-  NGHTTP3_STREAM_TYPE_UNKNOWN = UINT64_MAX,
-} nghttp3_stream_type;
+typedef uint64_t nghttp3_stream_type;
+
+#define NGHTTP3_STREAM_TYPE_CONTROL 0x00
+#define NGHTTP3_STREAM_TYPE_PUSH 0x01
+#define NGHTTP3_STREAM_TYPE_QPACK_ENCODER 0x02
+#define NGHTTP3_STREAM_TYPE_QPACK_DECODER 0x03
+#define NGHTTP3_STREAM_TYPE_UNKNOWN UINT64_MAX
 
 typedef enum nghttp3_ctrl_stream_state {
   NGHTTP3_CTRL_STREAM_STATE_FRAME_TYPE,
@@ -88,8 +88,8 @@ typedef struct nghttp3_varint_read_state {
 typedef struct nghttp3_stream_read_state {
   nghttp3_varint_read_state rvint;
   nghttp3_frame fr;
-  int state;
   int64_t left;
+  int state;
 } nghttp3_stream_read_state;
 
 /* NGHTTP3_STREAM_FLAG_NONE indicates that no flag is set. */
@@ -186,18 +186,17 @@ typedef struct nghttp3_stream_callbacks {
 } nghttp3_stream_callbacks;
 
 typedef struct nghttp3_http_state {
-  /* status_code is HTTP status code received.  This field is used
-     if connection is initialized as client. */
-  int32_t status_code;
   /* content_length is the value of received content-length header
      field. */
   int64_t content_length;
   /* recv_content_length is the number of body bytes received so
      far. */
   int64_t recv_content_length;
+  nghttp3_pri pri;
+  /* status_code is HTTP status code received.  This field is used
+     if connection is initialized as client. */
+  int32_t status_code;
   uint32_t flags;
-  /* pri is a stream priority produced by nghttp3_pri_to_uint8. */
-  uint8_t pri;
 } nghttp3_http_state;
 
 struct nghttp3_stream {
@@ -227,13 +226,12 @@ struct nghttp3_stream {
       /* outq_offset is write offset relative to the element at outq_idx
          in outq. */
       uint64_t outq_offset;
-      /* ack_offset is offset acknowledged by peer relative to the first
-         element in outq. */
+      /* ack_base is the number of bytes acknowledged by a remote
+         endpoint where the first element in outq is positioned at. */
+      uint64_t ack_base;
+      /* ack_offset is the number of bytes acknowledged by a remote
+         endpoint so far. */
       uint64_t ack_offset;
-      /* ack_done is the number of bytes notified to an application that
-         they are acknowledged inside the first outq element if it is of
-         type NGHTTP3_BUF_TYPE_ALIEN. */
-      uint64_t ack_done;
       uint64_t unscheduled_nwrite;
       nghttp3_stream_type type;
       nghttp3_stream_read_state rstate;
@@ -257,7 +255,7 @@ struct nghttp3_stream {
   };
 };
 
-nghttp3_objalloc_def(stream, nghttp3_stream, oplent);
+nghttp3_objalloc_decl(stream, nghttp3_stream, oplent)
 
 typedef struct nghttp3_frame_entry {
   nghttp3_frame fr;
@@ -284,7 +282,8 @@ void nghttp3_varint_read_state_reset(nghttp3_varint_read_state *rvint);
 void nghttp3_stream_read_state_reset(nghttp3_stream_read_state *rstate);
 
 nghttp3_ssize nghttp3_read_varint(nghttp3_varint_read_state *rvint,
-                                  const uint8_t *src, size_t srclen, int fin);
+                                  const uint8_t *begin, const uint8_t *end,
+                                  int fin);
 
 int nghttp3_stream_frq_add(nghttp3_stream *stream,
                            const nghttp3_frame_entry *frent);
@@ -293,13 +292,11 @@ int nghttp3_stream_fill_outq(nghttp3_stream *stream);
 
 int nghttp3_stream_write_stream_type(nghttp3_stream *stream);
 
-nghttp3_ssize nghttp3_stream_writev(nghttp3_stream *stream, int *pfin,
-                                    nghttp3_vec *vec, size_t veccnt);
+size_t nghttp3_stream_writev(nghttp3_stream *stream, int *pfin,
+                             nghttp3_vec *vec, size_t veccnt);
 
 int nghttp3_stream_write_qpack_decoder_stream(nghttp3_stream *stream);
 
-int nghttp3_stream_outq_is_full(nghttp3_stream *stream);
-
 int nghttp3_stream_outq_add(nghttp3_stream *stream,
                             const nghttp3_typed_buf *tbuf);
 
@@ -331,7 +328,7 @@ nghttp3_buf *nghttp3_stream_get_chunk(nghttp3_stream *stream);
 
 int nghttp3_stream_is_blocked(nghttp3_stream *stream);
 
-int nghttp3_stream_add_outq_offset(nghttp3_stream *stream, size_t n);
+void nghttp3_stream_add_outq_offset(nghttp3_stream *stream, size_t n);
 
 /*
  * nghttp3_stream_outq_write_done returns nonzero if all contents in
@@ -339,7 +336,11 @@ int nghttp3_stream_add_outq_offset(nghttp3_stream *stream, size_t n);
  */
 int nghttp3_stream_outq_write_done(nghttp3_stream *stream);
 
-int nghttp3_stream_add_ack_offset(nghttp3_stream *stream, uint64_t n);
+/*
+ * nghttp2_stream_update_ack_offset updates the last acknowledged
+ * offset to |offset|.
+ */
+int nghttp3_stream_update_ack_offset(nghttp3_stream *stream, uint64_t offset);
 
 /*
  * nghttp3_stream_is_active returns nonzero if |stream| is active.  In
@@ -393,4 +394,4 @@ int nghttp3_client_stream_uni(int64_t stream_id);
  */
 int nghttp3_server_stream_uni(int64_t stream_id);
 
-#endif /* NGHTTP3_STREAM_H */
+#endif /* !defined(NGHTTP3_STREAM_H) */
