@@ -22,16 +22,20 @@
  * OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
  * WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
-#ifndef NGTCP2_HTTP_TEST_H
-#define NGTCP2_HTTP_TEST_H
+#ifndef NGHTTP3_HTTP_TEST_H
+#define NGHTTP3_HTTP_TEST_H
 
 #ifdef HAVE_CONFIG_H
 #  include <config.h>
-#endif /* HAVE_CONFIG_H */
+#endif /* defined(HAVE_CONFIG_H) */
 
-void test_nghttp3_http_parse_priority(void);
-void test_nghttp3_sf_parse_item(void);
-void test_nghttp3_sf_parse_inner_list(void);
-void test_nghttp3_check_header_value(void);
+#define MUNIT_ENABLE_ASSERT_ALIASES
 
-#endif /* NGTCP2_CONN_TEST_H */
+#include "munit.h"
+
+extern const MunitSuite http_suite;
+
+munit_void_test_decl(test_nghttp3_http_parse_priority)
+munit_void_test_decl(test_nghttp3_check_header_value)
+
+#endif /* !defined(NGHTTP3_HTTP_TEST_H) */
