@@ -35,6 +35,6 @@
 
 extern const MunitSuite tnode_suite;
 
-munit_void_test_decl(test_nghttp3_tnode_schedule);
+munit_void_test_decl(test_nghttp3_tnode_schedule)
 
 #endif /* !defined(NGHTTP3_TNODE_TEST_H) */
